unit JKCaptionReader;

(*
   Author:  Johan Korten
   Company: JKSOFT Engineering
   Date:    12/29/2001 (version 1.0)
   Version: 1.0 for Delphi 6 (all versions)
   Email:   johan@jksoft.nl
   Suggestions and improvements always welcome!!!

   Language Codes according to ISO 639, Microsoft and Macintosh

   How to use:
   Put component on mainform and set desired language
   in your program (.dpr) file you might want to add: "TMainForm(MainForm).JKCaptionReader.ReadCaptions;"
   to activate the component at the appropriate moment...
   (without quotes, Mainform is the mainform with the component, JKCaptionReader is the defaultname of the component...)

   Known issue: the component only works correctly at runtime, for some stupid reason at design-time it gives back the delphi IDE captions

   Set the default language and run the project, you now have a "LngCnf.cfg" file with the default language and a "Language.lng" where "Language"
   is the default language (if you put it to Dutch the file will be named "Dutch.lng"...

   Then set the language property to the other languages you want to make a language file for and run the project again and again...
   Open the .lng file in notepad and apply the desired changes (quite simple but effective!)

   At runtime the default language will be started first (if you've added the line to the dpr file that is...)
   You can always change the language to the desired one by calling JKCaptionReader.ReadCaptions(Desires_Language)
   if you use the one parameter version of the call, the component will use the current language

   note: you'll have to ship the lng-files with you're project at this time.
   As soon as I have got spare time again I'll use a better way and put them in res-files
   or something like that...

   Good luck

*)

interface

uses
  Windows, Messages, SysUtils, Classes, TypInfo, Forms, dialogs, IniFiles;

type
  TLanguage = (Abkhazian, Afar, Afrikaans, Albanian, Amharic, Arabic, Armenian, Assamese, Aymara,
               Azerbaijani, Bashkir, Basque, Bengali, Bhutani, Bihari, Bislama, Breton,
               Bulgarian, Burmese, Byelorussian, Cambodian, Catalan, Cherokee, Chewa,
               Chinese_Simplified, Chinese_Traditional, Corsican, Croatian, Czech, Danish, Divehi,
               Dutch, Edo, English, Esperanto, Estonian, Faeroese, Farsi, Fiji, Finnish, Flemish,
               French, Frisian, Fulfulde, Galician, Gaelic_Scottish, Gaelic_Manx, Georgian,
               German, Greek, Greenlandic, Guarani, Gujarati, Hausa, Hawaiian, Hebrew, Hindi,
               Hungarian, Ibibio, Icelandic, Igbo, Indonesian, Interlingua, Interlingue,
               Inuktitut, Inupiak, Irish, Italian, Japanese, Javanese, Kannada, Kanuri, Kashmiri,
               Kazakh, Kinyarwanda, Kirghiz, Kirundi, Korean, Kurdish, Laothian,
               Latin, Latvian, Lingala, Lithuanian, Macedonian, Malagasy, Malay,
               Malayalam, Maltese, Maori, Marathi, Moldavian, Mongolian, Nauru, Nepali, Norwegian,
               Occitan, Oriya, Oromo, Papiamentu, Pashto, Polish,
               Portuguese, Punjabi, Quechua, Rhaeto_Romance, Romanian, Russian, Sami,
               Samoan, Sangro, Sanskrit, Serbian, Serbo_Croatian, Sesotho, Setswana, Shona, Sindhi,
               Sinhalese, Siswati, Slovak, Slovenian, Somali, Spanish, Sundanese, Swahili,
               Swedish, Syriac, Tagalog, Tajik, Tamazight, Tamil, Tatar, Telugu, Thai, Tibetan,
               Tigrinya, Tonga, Tsonga, Turkish, Turkmen, Twi, Uighur, Ukrainian, Urdu, Uzbek,
               Venda, Vietnamese, Volapuk, Welsh, Wolof, Xhosa, Yi, Yiddish, Yoruba, Zulu, Other);

  TJKCaptionReader = class(TComponent)
  private
    FFileName: string;
    FAutoLoad : boolean;
    ConfigFile : TInifile;
    FLanguage : TLanguage;
    function GetFilename : string;
    function GetLanguage : TLanguage;
  protected
    procedure Loaded; override;
    function LangToStr(language : TLanguage) : String;
    function StrToLang(language : string) : TLanguage;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure WriteCaptions;
    procedure ReadCaptions(language : TLanguage); overload;
    procedure ReadCaptions; overload;
  published
    property Autoload: boolean read FAutoLoad write FAutoLoad;
    property Language : TLanguage read GetLanguage write FLanguage;
  end;


implementation

function TJKCaptionReader.GetFilename : string;
begin
  result := FFileName;
end;

function TJKCaptionReader.GetLanguage : TLanguage;
begin
  FFileName := LangToStr(FLanguage) + '.lng';
  result := FLanguage;
end;

function TJKCaptionReader.LangToStr(language : TLanguage) : String;
begin
  case Language of
    Abkhazian   : result := 'Abkhazian';
    Afar   : result := 'Afar';
    Afrikaans   : result := 'Afrikaans';
    Albanian   : result := 'Albanian';
    Amharic   : result := 'Amharic';
    Arabic   : result := 'Arabic';
    Armenian   : result := 'Armenian';
    Assamese   : result := 'Assamese';
    Aymara   : result := 'Aymara';
    Azerbaijani   : result := 'Azerbaijani';
    Bashkir   : result := 'Bashkir';
    Basque   : result := 'Basque';
    Bengali   : result := 'Bengali (Bangla)';
    Bhutani   : result := 'Bhutani';
    Bihari   : result := 'Bihari';
    Bislama   : result := 'Bislama';
    Breton   : result := 'Breton';
    Bulgarian   : result := 'Bulgarian';
    Burmese   : result := 'Burmese';
    Byelorussian : result := 'Byelorussian (Belarusian)';
    Cambodian   : result := 'Cambodian';
    Catalan   : result := 'Catalan';
    Cherokee   : result := 'Cherokee';
    Chewa   : result := 'Chewa';
    Chinese_Simplified   : result := 'Chinese (Simplified)';
    Chinese_Traditional   : result := 'Chinese (Traditional)';
    Corsican   : result := 'Corsican';
    Croatian   : result := 'Croatian';
    Czech   : result := 'Czech';
    Danish   : result := 'Danish';
    Divehi   : result := 'Divehi';
    Dutch   : result := 'Dutch';
    Edo   : result := 'Edo';
    English   : result := 'English';
    Esperanto   : result := 'Esperanto';
    Estonian   : result := 'Estonian';
    Faeroese   : result := 'Faeroese';
    Farsi   : result := 'Farsi';
    Fiji   : result := 'Fiji';
    Finnish   : result := 'Finnish';
    Flemish   : result := 'Flemish';
    French   : result := 'French';
    Frisian   : result := 'Frisian';
    Fulfulde   : result := 'Fulfulde';
    Galician   : result := 'Galician';
    Gaelic_Scottish   : result := 'Gaelic (Scottish)';
    Gaelic_Manx   : result := 'Gaelic (Manx)';
    Georgian   : result := 'Georgian';
    German   : result := 'German';
    Greek   : result := 'Greek';
    Greenlandic   : result := 'Greenlandic';
    Guarani   : result := 'Guarani';
    Gujarati   : result := 'Gujarati';
    Hausa   : result := 'Hausa';
    Hawaiian   : result := 'Hawaiian';
    Hebrew   : result := 'Hebrew';
    Hindi   : result := 'Hindi';
    Hungarian   : result := 'Hungarian';
    Ibibio   : result := 'Ibibio';
    Icelandic   : result := 'Icelandic';
    Igbo   : result := 'Igbo';
    Indonesian   : result := 'Indonesian';
    Interlingua   : result := 'Interlingua';
    Interlingue   : result := 'Interlingue';
    Inuktitut   : result := 'Inuktitut';
    Inupiak   : result := 'Inupiak';
    Irish   : result := 'Irish';
    Italian   : result := 'Italian';
    Japanese   : result := 'Japanese';
    Javanese   : result := 'Javanese';
    Kannada   : result := 'Kannada';
    Kanuri   : result := 'Kanuri';
    Kashmiri   : result := 'Kashmiri';
    Kazakh   : result := 'Kazakh';
    Kinyarwanda   : result := 'Kinyarwanda (Ruanda)';
    Kirghiz   : result := 'Kirghiz';
    Kirundi  : result := 'Kirundi (Rundi)';
    Korean   : result := 'Korean';
    Kurdish   : result := 'Kurdish';
    Laothian   : result := 'Laothian';
    Latin   : result := 'Latin';
    Latvian   : result := 'Latvian (Lettish)';
    Lingala   : result := 'Lingala';
    Lithuanian   : result := 'Lithuanian';
    Macedonian   : result := 'Macedonian';
    Malagasy   : result := 'Malagasy';
    Malay   : result := 'Malay';
    Malayalam   : result := 'Malayalam';
    Maltese   : result := 'Maltese';
    Maori   : result := 'Maori';
    Marathi   : result := 'Marathi';
    Moldavian   : result := 'Moldavian';
    Mongolian   : result := 'Mongolian';
    Nauru   : result := 'Nauru';
    Nepali   : result := 'Nepali';
    Norwegian   : result := 'Norwegian';
    Occitan   : result := 'Occitan';
    Oriya   : result := 'Oriya';
    Oromo  : result := 'Oromo (Afan, Galla)';
    Papiamentu   : result := 'Papiamentu';
    Pashto   : result := 'Pashto (Pushto)';
    Polish   : result := 'Polish';
    Portuguese   : result := 'Portuguese';
    Punjabi   : result := 'Punjabi';
    Quechua   : result := 'Quechua';
    Rhaeto_Romance   : result := 'Rhaeto-Romance';
    Romanian   : result := 'Romanian';
    Russian   : result := 'Russian';
    Sami  : result := 'Sami (Lappish)';
    Samoan   : result := 'Samoan';
    Sangro   : result := 'Sangro';
    Sanskrit   : result := 'Sanskrit';
    Serbian   : result := 'Serbian';
    Serbo_Croatian   : result := 'Serbo-Croatian';
    Sesotho   : result := 'Sesotho';
    Setswana   : result := 'Setswana';
    Shona   : result := 'Shona';
    Sindhi   : result := 'Sindhi';
    Sinhalese   : result := 'Sinhalese';
    Siswati   : result := 'Siswati';
    Slovak   : result := 'Slovak';
    Slovenian   : result := 'Slovenian';
    Somali   : result := 'Somali';
    Spanish   : result := 'Spanish';
    Sundanese   : result := 'Sundanese';
    Swahili : result := 'Swahili (Kiswahili)';
    Swedish   : result := 'Swedish';
    Syriac   : result := 'Syriac';
    Tagalog   : result := 'Tagalog';
    Tajik   : result := 'Tajik';
    Tamazight   : result := 'Tamazight';
    Tamil   : result := 'Tamil';
    Tatar   : result := 'Tatar';
    Telugu   : result := 'Telugu';
    Thai   : result := 'Thai';
    Tibetan   : result := 'Tibetan';
    Tigrinya   : result := 'Tigrinya';
    Tonga   : result := 'Tonga';
    Tsonga   : result := 'Tsonga';
    Turkish   : result := 'Turkish';
    Turkmen   : result := 'Turkmen';
    Twi   : result := 'Twi';
    Uighur   : result := 'Uighur';
    Ukrainian   : result := 'Ukrainian';
    Urdu   : result := 'Urdu';
    Uzbek   : result := 'Uzbek';
    Venda   : result := 'Venda';
    Vietnamese   : result := 'Vietnamese';
    Volapuk   : result := 'Volapk';
    Welsh   : result := 'Welsh';
    Wolof   : result := 'Wolof';
    Xhosa   : result := 'Xhosa';
    Yi   : result := 'Yi';
    Yiddish   : result := 'Yiddish';
    Yoruba   : result := 'Yoruba';
    Zulu   : result := 'Zulu';
  else
    result := 'Other';
  end;
end;

function TJKCaptionReader.StrToLang(language : String) : TLanguage;
begin
  if language = 'Abkhazian' then result := Abkhazian else
  if language = 'Afar' then result := Afar else
  if language = 'Afrikaans' then result := Afrikaans else
  if language = 'Albanian' then result := Albanian else
  if language = 'Amharic' then result := Amharic else
  if language = 'Arabic' then result := Arabic else
  if language = 'Armenian' then result := Armenian else
  if language = 'Assamese' then result := Assamese else
  if language = 'Aymara' then result := Aymara else
  if language = 'Azerbaijani' then result := Azerbaijani else
  if language = 'Bashkir' then result := Bashkir else
  if language = 'Basque' then result := Basque else
  if language = 'Bengali (Bangla)' then result := Bengali else
  if language = 'Bhutani' then result := Bhutani else
  if language = 'Bihari' then result := Bihari else
  if language = 'Bislama' then result := Bislama else
  if language = 'Breton' then result := Breton else
  if language = 'Bulgarian' then result := Bulgarian else
  if language = 'Burmese' then result := Burmese else
  if language = 'Byelorussian (Belarusian)' then result := Byelorussian else
  if language = 'Cambodian' then result := Cambodian else
  if language = 'Catalan' then result := Catalan else
  if language = 'Cherokee' then result := Cherokee else
  if language = 'Chewa' then result := Chewa else
  if language = 'Chinese (Simplified)' then result := Chinese_Simplified else
  if language = 'Chinese (Traditional)' then result := Chinese_Traditional else
  if language = 'Corsican' then result := Corsican else
  if language = 'Croatian' then result := Croatian else
  if language = 'Czech' then result := Czech else
  if language = 'Danish' then result := Danish else
  if language = 'Divehi' then result := Divehi else
  if language = 'Dutch' then result := Dutch else
  if language = 'Edo' then result := Edo else
  if language = 'English' then result := English else
  if language = 'Esperanto' then result := Esperanto else
  if language = 'Estonian' then result := Estonian else
  if language = 'Faeroese' then result := Faeroese else
  if language = 'Farsi' then result := Farsi else
  if language = 'Fiji' then result := Fiji else
  if language = 'Finnish' then result := Finnish else
  if language = 'Flemish' then result := Flemish else
  if language = 'French' then result := French else
  if language = 'Frisian' then result := Frisian else
  if language = 'Fulfulde' then result := Fulfulde else
  if language = 'Galician' then result := Galician else
  if language = 'Gaelic (Scottish)' then result := Gaelic_Scottish else
  if language = 'Gaelic (Manx)' then result := Gaelic_Manx else
  if language = 'Georgian' then result := Georgian else
  if language = 'German' then result := German else
  if language = 'Greek' then result := Greek else
  if language = 'Greenlandic' then result := Greenlandic else
  if language = 'Guarani' then result := Guarani else
  if language = 'Gujarati' then result := Gujarati else
  if language = 'Hausa' then result := Hausa else
  if language = 'Hawaiian' then result := Hawaiian else
  if language = 'Hebrew' then result := Hebrew else
  if language = 'Hindi' then result := Hindi else
  if language = 'Hungarian' then result := Hungarian else
  if language = 'Ibibio' then result := Ibibio else
  if language = 'Icelandic' then result := Icelandic else
  if language = 'Igbo' then result := Igbo else
  if language = 'Indonesian' then result := Indonesian else
  if language = 'Interlingua' then result := Interlingua else
  if language = 'Interlingue' then result := Interlingue else
  if language = 'Inuktitut' then result := Inuktitut else
  if language = 'Inupiak' then result := Inupiak else
  if language = 'Irish' then result := Irish else
  if language = 'Italian' then result := Italian else
  if language = 'Japanese' then result := Japanese else
  if language = 'Javanese' then result := Javanese else
  if language = 'Kannada' then result := Kannada else
  if language = 'Kanuri' then result := Kanuri else
  if language = 'Kashmiri' then result := Kashmiri else
  if language = 'Kazakh' then result := Kazakh else
  if language = 'Kinyarwanda (Ruanda)' then result := Kinyarwanda else
  if language = 'Kirghiz' then result := Kirghiz else
  if language = 'Kirundi (Rundi)' then result := Kirundi else
  if language = 'Korean' then result := Korean else
  if language = 'Kurdish' then result := Kurdish else
  if language = 'Laothian' then result := Laothian else
  if language = 'Latin' then result := Latin else
  if language = 'Latvian (Lettish)' then result := Latvian else
  if language = 'Lingala' then result := Lingala else
  if language = 'Lithuanian' then result := Lithuanian else
  if language = 'Macedonian' then result := Macedonian else
  if language = 'Malagasy' then result := Malagasy else
  if language = 'Malay' then result := Malay else
  if language = 'Malayalam' then result := Malayalam else
  if language = 'Maltese' then result := Maltese else
  if language = 'Maori' then result := Maori else
  if language = 'Marathi' then result := Marathi else
  if language = 'Moldavian' then result := Moldavian else
  if language = 'Mongolian' then result := Mongolian else
  if language = 'Nauru' then result := Nauru else
  if language = 'Nepali' then result := Nepali else
  if language = 'Norwegian' then result := Norwegian else
  if language = 'Occitan' then result := Occitan else
  if language = 'Oriya' then result := Oriya else
  if language = 'Oromo (Afan, Galla)' then result := Oromo else
  if language = 'Papiamentu' then result := Papiamentu else
  if language = 'Pashto (Pushto)' then result := Pashto else
  if language = 'Polish' then result := Polish else
  if language = 'Portuguese' then result := Portuguese else
  if language = 'Punjabi' then result := Punjabi else
  if language = 'Quechua' then result := Quechua else
  if language = 'Rhaeto-Romance' then result := Rhaeto_Romance else
  if language = 'Romanian' then result := Romanian else
  if language = 'Russian' then result := Russian else
  if language = 'Sami (Lappish)' then result := Sami else
  if language = 'Samoan' then result := Samoan else
  if language = 'Sangro' then result := Sangro else
  if language = 'Sanskrit' then result := Sanskrit else
  if language = 'Serbian' then result := Serbian else
  if language = 'Serbo-Croatian' then result := Serbo_Croatian else
  if language = 'Sesotho' then result := Sesotho else
  if language = 'Setswana' then result := Setswana else
  if language = 'Shona' then result := Shona else
  if language = 'Sindhi' then result := Sindhi else
  if language = 'Sinhalese' then result := Sinhalese else
  if language = 'Siswati' then result := Siswati else
  if language = 'Slovak' then result := Slovak else
  if language = 'Slovenian' then result := Slovenian else
  if language = 'Somali' then result := Somali else
  if language = 'Spanish' then result := Spanish else
  if language = 'Sundanese' then result := Sundanese else
  if language = 'Swahili (Kiswahili)' then result := Swahili else
  if language = 'Swedish' then result := Swedish else
  if language = 'Syriac' then result := Syriac else
  if language = 'Tagalog' then result := Tagalog else
  if language = 'Tajik' then result := Tajik else
  if language = 'Tamazight' then result := Tamazight else
  if language = 'Tamil' then result := Tamil else
  if language = 'Tatar' then result := Tatar else
  if language = 'Telugu' then result := Telugu else
  if language = 'Thai' then result := Thai else
  if language = 'Tibetan' then result := Tibetan else
  if language = 'Tigrinya' then result := Tigrinya else
  if language = 'Tonga' then result := Tonga else
  if language = 'Tsonga' then result := Tsonga else
  if language = 'Turkish' then result := Turkish else
  if language = 'Turkmen' then result := Turkmen else
  if language = 'Twi' then result := Twi else
  if language = 'Uighur' then result := Uighur else
  if language = 'Ukrainian' then result := Ukrainian else
  if language = 'Urdu' then result := Urdu else
  if language = 'Uzbek' then result := Uzbek else
  if language = 'Venda' then result := Venda else
  if language = 'Vietnamese' then result := Vietnamese else
  if language = 'Volapk' then result := Volapuk else
  if language = 'Welsh' then result := Welsh else
  if language = 'Wolof' then result := Wolof else
  if language = 'Xhosa' then result := Xhosa else
  if language = 'Yi' then result := Yi else
  if language = 'Yiddish' then result := Yiddish else
  if language = 'Yoruba' then result := Yoruba else
  if language = 'Zulu' then result := Zulu else
  result := Other;
end;

constructor TJKCaptionReader.Create(AOwner: TComponent);
var taal : string;
begin
  inherited Create(AOwner);
  FAutoLoad := false;
  Configfile := TIniFile.Create(extractfilepath(paramstr(0)) + 'LngCnf.cfg');
  FLanguage := StrToLang(Configfile.ReadString('Config', 'Language', LangToStr(FLanguage)));
  FFileName := LangToStr(FLanguage) + '.lng';
end;

destructor TJKCaptionReader.Destroy;
begin
  Configfile.WriteString('Config', 'Language', LangToStr(FLanguage));
  configfile.Free;
  inherited Destroy;
end;

procedure TJKCaptionReader.Loaded;
begin
  inherited Loaded;
  if FAutoLoad then self.ReadCaptions(FLanguage);
end;

procedure TJKCaptionReader.WriteCaptions;
var
  RootCmp, XComponent: TComponent;
  MyForm : TForm;
  XPropInfo: PPropInfo;
  XCaption: string;
  k, i: integer;
  myinifile : tinifile;
begin
  FFileName := LangToStr(FLanguage) + '.lng';
  myinifile := tinifile.Create(extractfilepath(paramstr(0)) +  FFileName);
  try
    if Assigned(Owner) then
    begin
      RootCmp := Application;
      for k := 0 to RootCmp.ComponentCount-1 do
      begin
        if (RootCmp.Components[k] is TForm) then
        begin
          MyForm := TForm(RootCmp.Components[k]);
          myinifile.WriteString(MyForm.Name, 'Caption', MyForm.Caption);
          for i:=0 to MyForm.ComponentCount-1 do
          begin
            XComponent:=MyForm.Components[i];
            if Assigned(XComponent) then
            begin
              try
                XPropInfo:=GetPropInfo(XComponent, 'Caption');
                if Assigned(XPropInfo) then
                begin
                  XCaption:=GetStrProp(XComponent, 'Caption');
                  if (XComponent.Name <> '') and (XCaption <> '') and (XCaption <> '-') then
                    myinifile.WriteString(MyForm.Name, XComponent.Name, XCaption );
                end;
              except
                //
              end;
            end;
          end;
        end
        else
        begin
          XComponent:= RootCmp.Components[k];
          if Assigned(XComponent) then
          begin
            try
              XPropInfo:=GetPropInfo(XComponent, 'Caption');
              if Assigned(XPropInfo) then
              begin
                XCaption:=GetStrProp(XComponent, 'Caption');
                if (XComponent.Name <> '') and (XCaption <> '') and (XCaption <> '-') then
                  myinifile.WriteString(XComponent.Name, XComponent.Name, XCaption );
              end;
            except
              //
            end;
          end;
        end;
      end;
    end;
  finally
    myinifile.UpdateFile;
    myinifile.Free;
  end;
end;

procedure TJKCaptionReader.ReadCaptions;
begin
  FFileName := LangToStr(FLanguage) + '.lng';
  if not fileexists(FFileName) then
    WriteCaptions
  else
    ReadCaptions(self.FLanguage);
end;

procedure TJKCaptionReader.ReadCaptions(language : TLanguage);
var
  RootCmp, XComponent: TComponent;
  MyForm : TForm;
  XPropInfo: PPropInfo;
  value, section, XCaption: string;
  k, i: integer;
  myinifile : TIniFile;

begin
  FFileName := LangToStr(FLanguage) + '.lng';
  if not FileExists(FFileName) then
    exit
  else
    myinifile := tinifile.Create(extractfilepath(paramstr(0)) +  FFileName);
  try
    if Assigned(Owner) then
    begin
      RootCmp := Application;
      for k := 0 to RootCmp.ComponentCount-1 do
      begin
        if (RootCmp.Components[k] is TForm) then
        begin
          MyForm := TForm(RootCmp.Components[k]);
          myform.Caption := myinifile.ReadString(MyForm.Name, 'Caption', MyForm.Caption);
          section := MyForm.Name;
          for i:=0 to MyForm.ComponentCount-1 do
          begin
            XComponent:=MyForm.Components[i];
            if Assigned(XComponent) then
            begin
              try
                XPropInfo:=GetPropInfo(XComponent, 'Caption');
                if Assigned(XPropInfo) then
                begin
                  begin
                    XCaption := GetStrProp(XComponent, 'Caption');
                    value := myinifile.ReadString(section, XComponent.Name, XCaption);
                    SetStrProp(XComponent, 'Caption', value);
                  end;
                end;
              except
                //
              end;
            end;
          end;
        end;
      end;
    end;
  finally
    myinifile.Free;
  end;
end;

end.
